@@ -1,4 +1,4 @@
-# Copyright 2019 Cloudera Inc.
+# Copyright 2020 Cloudera Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -19,10 +19,14 @@ MASKING_CHARACTER <- "\U001"
 
 quote_chars <- c("\"", "'", "`")
 
+quote_char_regex <- "[\"'`']"
+
 ws_regex <- "[ \t\r\n]"
 
 word_char_regex <- "[[:alnum:]_.]"
 
+word_char_regex_with_colon <- "[[:alnum:]_.:]"
+
 non_word_char_regex <- "[^[:alnum:]_.]"
 
 digit_regex <- "[0-9]"
@@ -40,31 +44,64 @@ r_reserved_words <- c(
   # also the dots (..., ..1, ..2, and so on)
 )
 
+disallowed_names <- c(
+  # these names could cause ambiguity because SQL uses the dot for qualification
+  "is.na",
+  "is", "as", "na"
+)
+
+disallowed_aliases <- c(
+  # these aliases could cause ambiguity because SQL uses the dot for qualification
+  disallowed_names,
+  "character", "logical", "integer", "single", "double", "numeric",
+  "POSIXct", "datetime", "duration"
+)
+
 sql_characters_expecting_right_operands <- c(
   ">", "<", "=", "&", "|",
   "+", "-", "*", "/", "%",
-  "^", "!", "~"
+  "^", "!", "~", ":"
 )
 
 sql_words_expecting_right_operands <- c(
   "and", "or", "xor", "as", "from", "not",
   "in", "div", "between",
   "all", "any", "some",
-  "like", "ilike", "regexp", "iregexp", "rlike"
+  "like", "ilike", "regexp", "iregexp", "rlike",
+  "when", "then", "else"
 )
 
-sql_logical_operators_with_left_operands <- c(
-  "and", "or", "xor"
+sql_logical_operand_right_boundary_words <- c(
+  "and", "or", "xor",
+  "when", "then", "else", "end"
 )
 
-sql_logical_operators_with_right_operands <- c(
-  "and", "or", "xor", "not"
+sql_logical_operand_left_boundary_words <- c(
+  "and", "or", "xor", "not",
+  "case", "when", "then", "else"
 )
 
 sql_data_types_with_args <- c(
   "decimal", "char", "varchar"
 )
 
+sql_join_keywords <- c(
+  "on", "using",
+  "join",
+  "inner", "outer", "anti", "semi", "cross",
+  "left", "right", "full",
+  "natural"
+)
+
+sql_join_conditions_allowed_characters <- c(
+  "(", ")",
+  sql_characters_expecting_right_operands
+)
+
+sql_join_conditions_allowed_logical_operators <- c(
+  "not", "and", "or", "xor"
+)
+
 sql_reserved_words <- c(
   "add",
   "aggregate",
@@ -283,8 +320,12 @@ is_word_start_character <- function(char, useBytes = FALSE) {
   grepl(word_char_regex, char, useBytes = useBytes) && !grepl(digit_regex, char, useBytes = useBytes)
 }
 
-is_word_character <- function(char, useBytes = FALSE) {
-  grepl(word_char_regex, char, useBytes = useBytes)
+is_word_character <- function(char, useBytes = FALSE, allow_colons_in_word = FALSE) {
+  if (allow_colons_in_word) {
+    grepl(word_char_regex_with_colon, char, useBytes = useBytes)
+  } else {
+    grepl(word_char_regex, char, useBytes = useBytes)
+  }
 }
 
 is_non_word_character <- function(char, useBytes = FALSE) {
@@ -341,7 +382,7 @@ find_keyword_pairs <- function(expr_quotes_masked, keyword_1, keyword_2, operand
   keyword_pair_pos <- list()
 
   rc <- rawConnection(raw(0L), "r+")
-
+  on.exit(close(rc))
   writeChar(paste0(expr_quotes_masked, " "), rc)
   len <- seek(rc, 0L) - 1L
 
@@ -394,7 +435,6 @@ find_keyword_pairs <- function(expr_quotes_masked, keyword_1, keyword_2, operand
     }
 
   }
-  close(rc)
   keyword_pair_pos
 }
 
@@ -450,7 +490,7 @@ find_end_of_boolean_operand_after <- function(rc, len, in_parens) {
         return(pos + 1L)
       }
     } else if (in_parens == orig_parens &&
-               isTRUE(tolower(next_thing) %in% c(sql_logical_operators_with_left_operands, ","))) {
+               isTRUE(tolower(next_thing) %in% c(sql_logical_operand_right_boundary_words, ","))) {
       return(pos + 1L)
     }
   }
@@ -478,14 +518,14 @@ find_beginning_of_boolean_operand_before <- function(rc, in_parens) {
         return(pos + 1L)
       }
     } else if (in_parens == orig_parens &&
-               isTRUE(tolower(previous_thing) %in% c(sql_logical_operators_with_right_operands, ","))) {
+               isTRUE(tolower(previous_thing) %in% c(sql_logical_operand_left_boundary_words, ","))) {
       return(pos)
     }
   }
   return(0L)
 }
 
-get_next_character_word_or_number <- function(rc, len) {
+get_next_character_word_or_number <- function(rc, len, allow_colons_in_word = FALSE) {
   # get the next non-space character
   # or if it's a word character or digit, then get the whole word or number
   # or if there is nothing after, then return character(0)
@@ -511,8 +551,8 @@ get_next_character_word_or_number <- function(rc, len) {
     # if it's a word character or digit, keep going
     # until there's a non-word character or non-digit,
     # and return the whole word or number
-    if (is_word_character(char)) { # this matches digits
-      while(is_word_character(char)) { # this matches digits
+    if (is_word_character(char, allow_colons_in_word)) { # this matches digits
+      while(is_word_character(char, allow_colons_in_word)) { # this matches digits
         out_str <- paste0(out_str, char)
         char <- readChar(rc, 1L)
       }
@@ -526,7 +566,7 @@ get_next_character_word_or_number <- function(rc, len) {
   out_str
 }
 
-get_previous_character_word_or_number <- function(rc) {
+get_previous_character_word_or_number <- function(rc, allow_colons_in_word = FALSE) {
   # get the previous non-space character
   # or if it's a word character or digit, then get the whole word or number
   # or if there is nothing before, then return character(0)
@@ -553,8 +593,8 @@ get_previous_character_word_or_number <- function(rc) {
     # if it's a word character or digit, keep going
     # until there's a non-word character or non-digit,
     # and return the whole word or number
-    if (is_word_character(char)) { # this matches digits
-      while(is_word_character(char)) { # this matches digits
+    if (is_word_character(char, allow_colons_in_word)) { # this matches digits
+      while(is_word_character(char, allow_colons_in_word)) { # this matches digits
         out_str <- paste0(char, out_str)
         at_start <- tryCatch({
           seek(rc, -2L, "current")
@@ -615,6 +655,19 @@ ends_with_operator_expecting_right_operand <- function(expr, except = c()) {
   FALSE
 }
 
+is_join_keyword <- function(expr) {
+  isTRUE(tolower(expr) %in% sql_join_keywords)
+}
+
+is_not_join_condition_operator <- function(expr) {
+  !isTRUE(
+    tolower(expr) %in% c(
+      sql_join_conditions_allowed_characters,
+      sql_join_conditions_allowed_logical_operators
+    )
+  )
+}
+
 is_one_valid_r_name <- function(x) {
   names <- make.names(x)
   length(names) == 1 && names == x
